package com.xfactorstudio.ahawasi;





import java.net.*;
import java.io.*;
import java.util.*;
import java.net.InetAddress;
import java.net.Socket;

/*
 * Ahawasi.java
 *
 *
 * Copyright (C) 2001 Neeld Tanksley
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
/**
 * The main class of the Ahawasi server
 *
 * @author Neeld Tanksley
 * @version .91
 */

public class Ahawasi {
	static Vector serviceConfigs;
	static Vector closedServiceConfigs = new Vector();
	static final ServiceList serviceList= new ServiceList();
	private ServiceListServer serviceListServer;
	private Hashtable serverVariables;
	static boolean debug;
	static int serviceListPort;
	
	static AhawasiConfiguration configurator;
	
	static PrintWriter screenOut = new PrintWriter(System.out, true);
	//OutputStreams NEW
	static LogOutputStream out;
	static LogOutputStream err;
	static LogOutputStream log;
	
	
	static String configFile = "config/ahawasi.xml";
	static String logDir = "../logs/";
	static String accessLogName = "access.log";
	static String errorLogName = "error.log";
	static final String version = "Ahawasi .91";
	static final String author = "Neeld Tanksley";
	static final String authorEmail = "neeld@xfactorstudio.com";	
	static final String appName = "Ahawasi";






public static void main(String[] args){


//Properties p = System.getProperties();
//Enumeration e = p.keys();
//while (e.hasMoreElements()){
//	String element = (String)e.nextElement();
//	System.out.println(element + " = " + p.getProperty(element));
//}

	ServerSocket serverSocket = null;
	boolean listening = true;
	ClientThread tempClient;
	int currentNode;

	if(args.length != 0){
		configFile = args[0];
	}

	screenOut.println("\n " + appName + " " + version);
	screenOut.println(" Copyright 2000 " + author);
	screenOut.println(" " + authorEmail);
	
	Ahawasi xserver = new Ahawasi(configFile);
}

	public Ahawasi(){
		this(configFile);
	}
  
	public Ahawasi(String confDir){
		
	
		//Get XServer configuration
		configurator = new AhawasiConfiguration(confDir);

		//set Server Wide Variables
		this.setServiceListPort(configurator.getServiceListPort());
		this.setDebug(configurator.getDebug());
		this.setServiceConfigs(configurator.getServices());
		this.serverVariables = configurator.getServerVariables();
		
		
		//Get logs
		err = this.getLogOutputStream(configurator.getLogDir(),"error.log");
		out = this.getLogOutputStream(configurator.getLogDir(),"system.log");
		log = this.getLogOutputStream(configurator.getLogDir(),"access.log");

		//try to start room list server
		if (this.getServiceListPort() != 0){
			startServiceListServer();
		}else{
			screenOut.println("No or invalid port specified for RoomServer");
			screenOut.println("RoomServer disabled");
		}

		//Start services
		for(int i=0;i<serviceConfigs.size();i++){
			startService((XMLElement)serviceConfigs.get(i));
		}

		//start line command
		LineCommand tempCommand = new LineCommand(this);
		tempCommand.start();
	}



	/**
	*This method Creates the log if it does not exist
	*/
	public LogOutputStream getLogOutputStream(String logDir, String fileName){

		//make folders if not exist
		boolean blFolderExists = false;
		File LogFolder;
		File ErrorLog;
		FileWriter errorLog;

		LogFolder = new File(logDir);
		blFolderExists = LogFolder.exists();

		if (!blFolderExists){
			if (!LogFolder.mkdirs()){
				System.out.println("could not create log dir " + logDir);
				System.exit(-1);
			}
		}
		

		//Make file if not exisits
		boolean blFileExists = false;
		File logFile = new File(logDir + fileName);
		blFileExists = logFile.exists();
		if (!blFileExists){
			try{
				logFile.createNewFile();
			}catch(IOException e){
				System.out.println("Could not create " + logDir + fileName);
				System.exit(-1);
			}
		}
		
		LogOutputStream lops = null;
		try	{
			lops = new LogOutputStream(logDir + fileName);
		}catch(FileNotFoundException fnfe)	{
			System.out.println("could not create a LogOutputStream to " + logDir + fileName);
			fnfe.printStackTrace(System.out);
			System.exit(-1);
		}
		return lops;
	}

	/**
	* This method starts the service list server
	*/
	public void startServiceListServer(){
		serviceListServer = new ServiceListServer(this);
		serviceListServer.start();
	}
	
	/**
	* closes the service list server
	*/
	public void stopServiceListServer(){
		serviceListServer.close();
	}
	
	/**
	* restarts the service list server
	*/
	public void restartServiceListServer(){
		this.stopServiceListServer();
		this.startServiceListServer();
	}

	/**
	* starts a new Service Thread as defined by the configuration XML
	*/
	public void startService(XMLElement confXML){
		SocketService tempRoom = new SocketService(confXML,this);
		tempRoom.start();
	}

	/**
	* returns the service running on the given port number
	*/
	public SocketService getService(int port){
		SocketService tmpService = null;
		for(int i=0;i<serviceList.size();i++){
			tmpService = (SocketService)serviceList.get(i);
			if(tmpService.getPort() == port){
			break;
		}
	}
		return tmpService;
	}

	/**
	* Closes a service on the specified port
	*/
	public void stopService(int port){
		boolean foundRoom = false;
		for (int i=0;i<serviceList.size();i++){
			SocketService serviceFoo = (SocketService)serviceList.get(i);
			if(serviceFoo.getPort() == port){
				this.out.println("closing room: " + serviceFoo.getServiceName());
				serviceFoo.close();
				foundRoom = true;
				break;
			}
		}
		if(!foundRoom){
			this.err.println("could not find specified room");
		}
	}

	/**
	* This method prints dubugg messages to the console
	*/
	public void printDebugg(String message){
		if (debug){
			System.out.println(message);
		}
	}

	/**
	* If a Service is currently running on the specified port
	* this returns true, if not, false.
	*/
	public boolean serviceExists(int port){
		boolean foundRoom = false;
		for (int i=0;i<serviceList.size();i++){
			SocketService roomFoo = (SocketService)serviceList.get(i);
			if(roomFoo.getPort() == port){
				foundRoom = true;
				break;
			}
		}
		return foundRoom;
	}


	/**
	* Removes the client identified by userId
	*
	* @param userId the ID of the client as available from ClientThread.getUserId
	*/
	public void removeClient(int userId){
		for (int i=0;i<serviceList.size();i++){
			SocketService serviceFoo = (SocketService)serviceList.get(i);
			serviceFoo.removeClient(userId);
		}
	}

	/**
	* return the version of the server
	*/
	public String getVersion(){
		return this.version;
	}

	/**
	* return the author
	*/
	public String getAuthor(){
		return this.author;
	}

	/**
	* returns true if screen debugging is on, else false
	*/
	public boolean getDebug(){
		return this.debug;
	}

	/**
	* returns true if access logging is on, else false
	*/
	//public boolean getAccessLog(){
	//	return logWriter.getLogAccess() ;
	//}

	/**
	* returns the current service list port
	*/
	public int getServiceListPort(){
		return this.serviceListPort;
	}

	/**
	* set the port for service list to run on
	*/
	public void setServiceListPort(int serviceListPort){
		this.serviceListPort = serviceListPort;
	}

	/**
	* returns a ServiceList of current services
	*/
	public ServiceList getServiceList(){
		return this.serviceList;
	}

	/**
	* set state of debugging
	*/
	public void setDebug(boolean debug){
		this.debug = debug;
	}

	/**
	* set service configs
	*/
	public void setServiceConfigs(Vector serviceConfigs){
		this.serviceConfigs = serviceConfigs;
	}

	/**
	* return the current LogWriter
	*/
	//public LogWriter getLogWriter(){
	//	return this.logWriter;
	//}

	/**
	* return the serverVariable Vector
	*/
	public Hashtable getServerVariables(){
		return this.serverVariables;
	}

	/**
	* add the Service config XML to the list of closed rooms
	*/
	public void setClosedServiceConfigs(XMLElement serviceConf){
		this.closedServiceConfigs.add(serviceConf);
	}

	/**
	* return a list of closed services
	*/
	public Vector getClosedServiceConfigs(){
		return this.closedServiceConfigs;
	}
	 
	 public void shutDown() {
	 	this.stopServices();
		System.exit(0);
	 }

	/**
	* Closes all services
	*/
	public void stopServices(){
		for (int i=0;i<serviceList.size();i++){
			SocketService serviceFoo = (SocketService)serviceList.get(i);
				this.out.println("closing room: " + serviceFoo.getServiceName());
				serviceFoo.close();
		}
	}
}// end main




