package com.xfactorstudio.ahawasi;

import java.io.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import com.xfactorstudio.ahawasi.exceptions.*;

/*
 * AhawasiConfiguration.java
 *
 *
 * Copyright (C) 2001 Neeld Tanksley
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
/**
 * This class handles the configuration of the server
 *
 * @author Neeld Tanksley
 * @version .91
 */

public class AhawasiConfiguration {

  private boolean debug = false;
  private boolean logAccess = true;
  private boolean logError = true;
  private int serviceListPort = 0;
  private boolean serviceListDetail = false;
  private String logDir = "./logs";
  private Vector serviceConfig = new Vector();
  private Hashtable serverVariables = new Hashtable();
  private XMLElement xserverConfig = new XMLElement();

  public AhawasiConfiguration(String confFile) {

    xserverConfig = this.loadConf(confFile);

    //get all the children
    Vector childNodes = xserverConfig.getChildren();

    //First get the values of any attributes
    if(xserverConfig.getProperty("service_list_port") != null){
      setServiceListPort(Integer.parseInt(xserverConfig.getProperty("service_list_port")));
    }
    if(xserverConfig.getProperty("debug") != null){
     setDebug(Boolean.valueOf(xserverConfig.getProperty("debug")).booleanValue());
    }
    if(xserverConfig.getProperty("access_log") != null){
      setLogAccess(Boolean.valueOf(xserverConfig.getProperty("access_log")).booleanValue());
    }
    if(xserverConfig.getProperty("error_log") != null){
      setLogError(Boolean.valueOf(xserverConfig.getProperty("error_log")).booleanValue());
    }

    //Now check for any child nodes
    for(int i = 0;i<childNodes.size();i++){

     XMLElement fooNode = (XMLElement)childNodes.get(i);
     String fooNodeName = fooNode.getTagName();
     String fooNodeValue = fooNode.getContents();

     if(fooNodeName.equalsIgnoreCase("service")){
      serviceConfig.add(fooNode);
     }
     if(fooNodeName.equalsIgnoreCase("service_list_port")){
      setServiceListPort(Integer.parseInt(fooNodeValue));
     }
	 if(fooNodeName.equalsIgnoreCase("log_dir")){
	 	setLogDir(fooNodeValue);
     }
	 if(fooNodeName.equalsIgnoreCase("service_list_detail")){
      setServiceListDetail(Boolean.valueOf(fooNodeValue).booleanValue());
     } 
     if(fooNodeName.equalsIgnoreCase("debug")){
      setDebug(Boolean.valueOf(fooNodeValue).booleanValue());
     }
     if(fooNodeName.equalsIgnoreCase("access_log")){
      setLogAccess(Boolean.valueOf(fooNodeValue).booleanValue());
     }
     if(fooNodeName.equalsIgnoreCase("error_log")){
      setLogError(Boolean.valueOf(fooNodeValue).booleanValue());
     }
     if(fooNodeName.equalsIgnoreCase("server_variable")){
      String fooVariableName = fooNode.getProperty("name");
      setServerVariable(fooVariableName,fooNodeValue);
     }

    }


   }
  public void setServerVariable(String name,String value){
    serverVariables.put(name,value);
  }
  public Hashtable getServerVariables(){
   return this.serverVariables;
  }
  public void setServiceListPort(int serviceListPort){
    this.serviceListPort = serviceListPort;
  }
  public void setDebug(boolean debug){
    this.debug = debug;
  }
  public void setLogAccess(boolean logAccess){
    this.logAccess = logAccess;
  }
  public void setLogDir(String logDir){
    this.logDir = logDir;
  }
  public int getServiceListPort(){
    return this.serviceListPort;
  }
  public boolean getDebug(){
    return this.debug;
  }
  public boolean getLogAccess(){
    return this.logAccess;
  }
  public void setLogError(boolean logError){
     this.logError = logError;
  }
  public boolean getLogError(){
   return this.logError;
  }
  public String getLogDir(){
    return this.logDir;
  }
  public String toString(){
    return xserverConfig.toString();
  }
  public Vector getServices(){
    return serviceConfig;
  }
  
  public boolean getServiceListDetail(){
  	return this.serviceListDetail;
  }
  public void setServiceListDetail(boolean serviceListDetail){
	this.serviceListDetail = serviceListDetail;
  }

  public XMLElement loadConf(String confFile){
      XMLElement foo = new XMLElement();
      String xml = "";
      try{
       InputStream in = new FileInputStream(confFile);
          while (in.available() > 0) {
              byte[] b = new byte[in.available()];
              in.read(b);
              xml = xml + new String(b);
          }
          in.close();
          foo.parseString(xml, 0);
        }catch(FileNotFoundException e){
		  e.printStackTrace(System.out);
		  System.exit(-1);
        }catch(IOException e){
			e.printStackTrace(System.out);
			System.exit(-1);
        }
        return foo;
  }
}