package com.xfactorstudio.ahawasi;

import com.xfactorstudio.ahawasi.exceptions.*;
import java.net.*;
import java.io.*;
import java.util.*;

/*
 * ClientThread.java
 *
 *
 * Copyright (C) 2001 Neeld Tanksley
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
/**
 * The thread for a client
 *
 * @author Neeld Tanksley
 * @version .91
 */

public class ClientThread extends Thread {
    private MessageHandler handler;
    private SocketService service;
	private Socket socket = null;
	private Vector clientList;
	public PrintWriter out;
	public BufferedReader in;
    private boolean listening = true;
    private Ahawasi server;
	private String userName = "anonymous";
    private String messageHandler;
	private long  userId = Math.round(Math.random() * 100000000);
    private Response response;
    private Request request;
    private Hashtable sessionVariables = new Hashtable();
	private long totalBytesRecived = 0;
	private long totalBytesSent = 0;
	private char messageDelimiter;
	


	public ClientThread(Socket socket,
						ClientList clientList,
						String messageHandler,
						SocketService parent,
						Ahawasi server){

		this.service = parent;
		this.messageHandler = messageHandler;
		this.socket = socket;
		this.clientList = clientList;
		this.server = server;
		this.messageDelimiter = '\u0000';

	}
	
	/**
	* returns the current username of this client
	*/
	public String getUserName(){
		return this.userName;
	}
	
	/**
	* sets the current username of this client
	*/
	public void setUserName(String userName){
		this.userName = userName;
	}
	
	/**
	* returns the userId of this client
	*/
	public long getUserId(){
		return this.userId;
	}
	
	/**
	* returns the client IP address
	*/
	public String getClientIP(){
		return socket.getInetAddress().getHostAddress();
	}
	
	/**
	* returns the current session variables
	*/
	public Hashtable getSessionVariables(){
		return this.sessionVariables;
	}
	
	/**
	* kills this client thread
	*/
	public void kill(){
		this.listening = false;
		try	{
			out.close();
			in.close();
		}catch(IOException ioe){
			ioe.printStackTrace(new PrintStream(server.err));
		}
	}
	 
	/**
	* The main method of this thread
	*/
	public void run(){
		server.printDebugg("CONNECT: " + getClientIP());
		server.log.println("Connect -- " + getClientIP());
	
		try{
			out = new PrintWriter(socket.getOutputStream(), true);
			in = new BufferedReader (new InputStreamReader(socket.getInputStream()));
			try{
				//get response / request
				request = new Request(server,service,this);
				response = new Response(server,service,this);
				//get Instance of Handler
				handler = (MessageHandler)Class.forName(messageHandler).newInstance();
				handler.setRequest(request);
				handler.setResponse(response);
			}catch(ClassNotFoundException e){
				e.printStackTrace(new PrintStream(server.err));
			}catch(InstantiationException e){
				e.printStackTrace(new PrintStream(server.err));
			}catch(IllegalAccessException e){
				e.printStackTrace(new PrintStream(server.err));
			}

			handler.onConnect();
			char buf[] = new char[1];
			
			//ENTER LISTENING / MESSAGE READING LOOP
			while (in.read(buf, 0 ,1)!=-1 && listening == true){
				StringBuffer inputLine = new StringBuffer();
				while(buf[0]!=messageDelimiter){
					totalBytesRecived++;
					if (inputLine.length() < service.getMaxMessageSize()){
						inputLine.append(buf[0]);
						in.read(buf, 0 ,1);
					}else{
						throw new BufferOverflowException("maximum message size of " + service.getMaxMessageSize() + " exceeded");
					}
				}
				//don't send potentially incomplete messages
				if (listening){
					handler.onMessage(inputLine.toString());
					server.printDebugg("<<<< " + this.getUserId() + " " + this.getClientIP() + " " + inputLine);
				}
			}
			
		}catch(IOException e){
			//this catches mostly errors writing to the client stream
			//when the client has dissconnected. Best to leave this commented out
			//or the logs will fill up.
			e.printStackTrace(new PrintStream(server.err));
		}catch(NullPointerException e){
			e.printStackTrace(new PrintStream(server.err));
		}catch(BufferOverflowException boe){
			boe.printStackTrace(new PrintStream(server.err));
		}finally {
			handler.onDisconnect();
			service.removeClient(this);
			try{
				in.close();
				out.close();
				socket.close();
			}catch(IOException e){
				e.printStackTrace(new PrintStream(server.err));
			}

		}
	}
	/**
	* Sends the specified message the this client
	*/
	public void send(String message){
		if (out != null){
			totalBytesSent = message.length();
			out.print(message + this.messageDelimiter);
			out.flush();
			server.printDebugg(">>>> " + this.getUserId() + " " + this.getClientIP() + " " + message);
		}
	}
	
	
	public long getTotalBytesSent(){
		return this.totalBytesSent;
	}
	public long getTotalBytesRecieved(){
		return this.totalBytesRecived;
	}
	
	
	public String toXML(){
		String tmpXML = "";
		tmpXML += "<client userid=\"" + userId + "\">";
		tmpXML += "<listening>" + listening + "</listening>";
		tmpXML += "<ipaddress>" + this.getClientIP() + "</ipaddress>";
		tmpXML += "<username>" + userName + "</username>";
		tmpXML += "<bytesrecived>" + totalBytesRecived + "</bytesrecived>";
		tmpXML += "<bytessent>" + totalBytesSent + "</bytessent>";
		tmpXML += "<session>";
		if (!sessionVariables.isEmpty()){
			Enumeration sKeys = sessionVariables.keys();
			while (sKeys.hasMoreElements()){
				String myKey = (String)sKeys.nextElement();
				tmpXML += "<var>";
				tmpXML += "<name>" + myKey + "</name>";
				tmpXML += "<value>" + sessionVariables.get(myKey) + "</value>";
				tmpXML += "</var>";
			}
		}		
		tmpXML += "</session>";
		tmpXML += "</client>";
		
		return tmpXML;
	}
	
}