package com.xfactorstudio.ahawasi;

import java.io.*;
import java.util.*;


/*
 * LineCommand.java
 *
 *
 * Copyright (C) 2001 Neeld Tanksley
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 

/**
 *
 *
 * The line command interface
 *
 * @author Neeld Tanksley
 * @version .91
 */

public class LineCommand extends Thread{
	private BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
	private PrintWriter screenOut = new PrintWriter(System.out, true);
	private String command;
	private int activeRoom = 0;
    private Ahawasi server;
    private ServiceList serviceList;


	public LineCommand(Ahawasi server){
          this.server = server;
          this.serviceList = server.getServiceList();
	}

	private void setActiveRoom(int newPort){
		activeRoom = newPort;
	}
	private boolean isValidPort(String testPort){
		boolean validPort = false;
		int tmpPort = 0;
		try{
			tmpPort = Integer.parseInt(testPort);
			validPort = true;
		}catch(Exception e){
			validPort = false;
		}
		return validPort;
	}
/**
 * Prints to screen a listing of information about the room specified by port.
 * If port is 0, then it shows information about the server.
 */
public void showRoomInfo(PrintWriter out, int port){
    if(port == 0){
        out.println("___________________SERVER INFO________________________");
        out.println("VERSION:          " + server.getVersion());
        out.println("AUTHOR:           " + server.getAuthor());
        out.println("DEBUG:            " + server.getDebug());
        //out.println("ACCESS_LOG:       " + server.getAccessLog());
        out.println("SERVICELIST_PORT: " + server.getServiceListPort());
        out.println();
        out.println("___________________SERVER VARIABLES____________________");
        Hashtable serverVariables = server.getServerVariables();
        Enumeration serverVariableKeys = serverVariables.keys();
        String thisKey;
        while(serverVariableKeys.hasMoreElements()){
          thisKey = (String)serverVariableKeys.nextElement();
         out.println(thisKey + " = " + serverVariables.get(thisKey));
        }
    }else{
      for (int i=0;i<serviceList.size();i++){
        SocketService serviceFoo = (SocketService)serviceList.get(i);
        if(serviceFoo.getPort() == port){
                out.println("___________________SERVICE INFO________________________");
                out.println("ROOM_NAME:       " + serviceFoo.getServiceName());
                out.println("PORT:            " + serviceFoo.getPort());
                out.println("MAX_USERS:       " + serviceFoo.getMaxUsers());
                out.println("CURRENT_USERS:   " + serviceFoo.getClientList().size());
                out.println();
                out.println("__________________APPLICATION VARIABLES__________________");
                Hashtable applicationVariables = serviceFoo.getApplicationVariables();
                Enumeration applicationVariableKeys = applicationVariables.keys();
                String thisKey;
                while(applicationVariableKeys.hasMoreElements()){
                  thisKey = (String)applicationVariableKeys.nextElement();
                 out.println(thisKey + " = " + applicationVariables.get(thisKey));
                }
        }
      }
    }
}
public void printRoomListScreen(PrintWriter out){
		//Vector tmpRoomList = roomList;
		String spacing = "";
		int tmpLength = 0;
		out.println("    PORT    |        SERVICE NAME          ");
		out.println("__________________________________________________________");
		for (int i=0;i<serviceList.size();i++){
			spacing = "";
			SocketService serviceFoo = (SocketService)serviceList.get(i);
			if(serviceFoo.getPort() > 100){tmpLength = 3;}
			if(serviceFoo.getPort() > 1000){tmpLength = 4;}
			if(serviceFoo.getPort() > 10000){tmpLength = 5;}
			if(serviceFoo.getPort() > 100000){tmpLength = 6;}
			tmpLength = 12 - tmpLength;
			for(int j=0;j<tmpLength;j++){
				spacing += " ";
			}
			out.println(serviceFoo.getPort() + spacing + "|" +  serviceFoo.getServiceName());
		}
}
public void printClientList(PrintWriter out){
		for (int i=0;i<serviceList.size();i++){
			SocketService serviceFoo = (SocketService)serviceList.get(i);
                        out.println();
			out.println("SERVICE: " + serviceFoo.getPort() + " " + serviceFoo.getServiceName());
			out.println("___________________________________________________");
			out.println(showClientsScreen(serviceFoo.getClientList()));
		}

}
public String showClientsScreen(Vector clientList){
	String tmpClients = "";
	boolean roomEmpty = true;
	for (int i=0;i<clientList.size();i++){
		roomEmpty = false;
		ClientThread foo = (ClientThread)clientList.get(i);
		tmpClients += "     CLIENT: " + foo.getUserId() +  " " + foo.getClientIP() + " " + foo.getUserName() + "\n";
                tmpClients += "     ______________________________________________\n";
                Hashtable sessionVariables = foo.getSessionVariables();

                //sessionVariables.put("test 1","answer 1");
                //sessionVariables.put("test 2","answer 2");
                //sessionVariables.put("test 3","answer 3");
                //sessionVariables.put("test 4","answer 4");
                //sessionVariables.put("test 5","answer 5");
                //sessionVariables.put("test 6","answer 6");

                Enumeration sessionVariableKeys = sessionVariables.keys();
                String thisKey;
                while(sessionVariableKeys.hasMoreElements()){
                  thisKey = (String)sessionVariableKeys.nextElement();
                  tmpClients += "     " + thisKey + " = " + sessionVariables.get(thisKey) + "\n";
                }
                tmpClients += "     ______________________________________________\n";
	}
	if(roomEmpty){
			tmpClients += "empty\n";
	}
	return tmpClients;
}

public boolean doHelp(String command){
  screenOut.println("QUIT");
  screenOut.println("DEBUG [true/false]");
  screenOut.println("SERVICES");
  //screenOut.println("SERVICE <stop/start/restart> <port>");
  screenOut.println("SERVICELIST <stop/start/restart>");
  screenOut.println("INFO [port]");
  screenOut.println("CLIENTS");
  screenOut.println("KILLCLIENT <clientId>");


  screenOut.println("");
  return true;
}
public void run(){
        boolean valid;

        // Pause the Line Command while rooms start
        try{
                this.sleep(2000);
        }catch(InterruptedException e){
                e.printStackTrace();
        }

        try{

          screenOut.println("");
          screenOut.print("Ahawasi>");
          screenOut.flush();
          while ((command = in.readLine()) != null){
            valid = false;
            command = command.trim().toUpperCase();
            screenOut.println();

/*
            if (command.compareTo("HELP")==0){
            valid = this.doHelp(command);
              //Ahawasi.debug=false;
            }
**/
            //STOP SERVICE <port>
            if (command.startsWith("SERVICE STOP")){
              boolean validPort = false;
              int tmpPort = 0;
              if(command.length() > 13){
                try{
                  tmpPort = Integer.parseInt(command.substring(13,command.length()));
                  validPort = true;
                }catch(Exception e){
                  validPort = false;
                }
              }
              if (validPort){
                server.stopService(tmpPort);
              }else{
                screenOut.println("you seem to have specified an invalid port");
                screenOut.println("STOP SERVICE <port>");
              }
              valid = true;
            }
/**
            //RESTART SERVICE <port>
            if (command.startsWith("SERVICE RESTART")){
              boolean validPort = false;
              int tmpPort = 0;
              if(command.length() > 16){
                try{
                  tmpPort = Integer.parseInt(command.substring(16,command.length()));
                  validPort = true;
                }catch(Exception e){
                  validPort = false;
                }
              }
              if (validPort){
                server.stopService(tmpPort);
                Vector closedServices = server.getClosedServiceConfigs();
                for(int i=0;i<closedServices.size();i++){
                  server.startService((XMLElement)closedServices.get(i));
                  closedServices.remove(i);
                }

              }else{
                screenOut.println("you seem to have specified an invalid port");
                screenOut.println("STOP SERVICE <port>");
              }
              valid = true;
            }
            //USE
              if (command.startsWith("USE")){
                String portToUse = "";
                if(command.length() > 4){
                  portToUse = command.substring(4,command.length());
                }
                if(isValidPort(portToUse)){
                  if(server.serviceExists(Integer.parseInt(portToUse))){
                    setActiveRoom(Integer.parseInt(portToUse));
                    screenOut.println("using: " + portToUse.trim());
                  }else{
                    screenOut.println("that room does not exist");
                  }
                }else{
                  screenOut.println("you seem to have specified an invalid port");
                }
                valid = true;
              }

 */
                //GUI
                //if (command.compareTo("GUI")==0){
                //        gui = new AhawasiGUI(server);
                //        valid = true;
                //}
                //INFO
              if (command.startsWith("INFO")){
                String portToUse = "";
                if(command.length() > 5){
                  portToUse = command.substring(5,command.length());
                }else{
                  this.showRoomInfo(screenOut,0);
                  valid = true;
                }
                if(isValidPort(portToUse)){
                  if(server.serviceExists(Integer.parseInt(portToUse))){
                    this.showRoomInfo(screenOut,Integer.parseInt(portToUse));
                  }else{
                    screenOut.println("that room does not exist");
                  }
                  valid = true;
                }
                if (!valid){
                  screenOut.println("you seem to have specified an invalid port");
                }
                valid = true;
              }
                //CLIENTS
              if (command.compareTo("CLIENTS")==0){
                      printClientList(screenOut);
                      valid = true;
              }
              //KILLCLIENT <ID>
              if (command.startsWith("KILLCLIENT")){
                      String clientId = "";
                      if(command.length() > 11){
                              clientId = command.substring(11,command.length());
                      }
                      server.removeClient(Integer.parseInt(clientId));
                      valid = true;
              }
              //SERVICES
              if (command.compareTo("SERVICES")==0){
                      printRoomListScreen(screenOut);
                      valid = true;
              }
               //DEBUG
              if (command.compareTo("DEBUG")==0){
                      screenOut.println("debug is currently " + Ahawasi.debug);
                      valid = true;
              }
              //DEBUG FALSE
              if (command.compareTo("DEBUG FALSE")==0){
                      screenOut.println("debug disabled");
                      valid = true;
                      Ahawasi.debug=false;
              }
                //DEBUG TRUE
              if (command.compareTo("DEBUG TRUE")==0){
                      screenOut.println("debug enabled");
                      valid = true;
                      Ahawasi.debug=true;
              }
                //SERVICELIST STOP
              if(command.compareTo("SERVICELIST STOP")==0){
                valid = true;
                server.stopServiceListServer();
              }
              //SERVICE LIST START
              if(command.compareTo("SERVICELIST START")==0){
                valid = true;
                server.startServiceListServer();
              }
              //SERVICELIST RESTART
              if(command.compareTo("SERVICELIST RESTART")==0){
                valid = true;
                server.restartServiceListServer();
              }
                //QUIT
              if (command.compareTo("QUIT")==0){
                      screenOut.println("Ahawasi shutting down");
                      valid = true;
                      System.exit(0);
              }
              //See if it was a valie command
              if (!valid){
                screenOut.println("try quit");
                      //screenOut.println("Unknown command");
                      //screenOut.println("try HELP");
              }
              //pause before showing prompt
              try{
                this.sleep(200);
              }catch(InterruptedException e){
                e.printStackTrace();
              }
              //show prompt
                screenOut.print("\nAhawasi>");
                screenOut.flush();

            }//end while

        }catch (IOException e){
          e.printStackTrace();
      }
   }
}