package com.xfactorstudio.ahawasi;
import java.util.Hashtable;
/*
 * Request.java
 *
 *
 * Copyright (C) 2001 Neeld Tanksley
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 

/**
 *
 *
 * The Request object provided to a message hander
 *
 * @author Neeld Tanksley
 * @version .91
 */

public class Request {
  private SocketService service;
  private ClientThread client;
  private Ahawasi server;


  public Request(Ahawasi server,SocketService service,ClientThread client) {
    this.service = service;
    this.client = client;
    this.server = server;
  }
  
	/**
	* return the clientList for current service
	*/
	public ClientList getClientList(){
		return service.getClientList();
	}
	
	public SocketService getService(){
		return this.service;
	}
	
	public SocketService getService(int port){
		return server.getService(port);
	}
	
	public ClientThread getClient(int port,long uid){
		return server.getService(port).getClient(uid);
	}
	
	public ClientThread getClient(){
		return this.client;
	}
	
	/**
	*	return the clientList of the service 
	*	specified by the port number
	**/
	public ClientList getClientList(int port){
		ServiceList serviceList = server.getServiceList();
		SocketService ss = null;
		
		for(int i=0;i<serviceList.size();i++){
			ss = (SocketService)serviceList.get(i);
			if (ss.getPort() == port){
				return ss.getClientList();
			}
		}
		return null;
	}

	/**
	* return the ID of this user
	*/
	public long getUserID(){
		return client.getUserId();
	}	

	/**
	* returns the IP address of the this client
	*/
	public String getClientIP(){
		return client.getClientIP();
	}

	/**
	* returns a clients IP address given a userID
	* if the id is not found, returns null
	*/
	public String getClientIP(long userID){
		ClientList clientList = service.getClientList();
		ClientThread tempClient;
		for(int i=0;i<clientList.size();i++){
			tempClient = (ClientThread)clientList.get(i);
			if(tempClient.getUserId() == userID){
				return tempClient.getClientIP();
			}
		}
		return null;
	}

	/**
	* returns a clients IP address given a username
	* if the name is not found, returns null
	*/
	public String getClientIP(String userName){
		ClientList clientList = service.getClientList();
		ClientThread tempClient;
		for(int i=0;i<clientList.size();i++){
			tempClient = (ClientThread)clientList.get(i);
			if(tempClient.getUserName() == userName){
				return tempClient.getClientIP();
			}
		}
		return null;
	}

	/**
	*	Returns this users name
	**/
	public String getUserName(){
		return client.getUserName();
	}
	
	/**
	* returns a clients IP address given a userID
	* if the id is not found, returns null
	*/
	public String getUserName(long userId){
		ClientList clientList = service.getClientList();
		ClientThread tempClient;
		for(int i=0;i<clientList.size();i++){
			tempClient = (ClientThread)clientList.get(i);
			if(tempClient.getUserId() == userId){
				return tempClient.getUserName();
			}
		}
		return null;
	}

	public String getServerVariable(String varName){
		Hashtable tmpServerVars = server.getServerVariables();
		return (String)tmpServerVars.get(varName);
	}

	public String getApplicationVariable(String varName){
		Hashtable tmpServiceVars = service.getApplicationVariables();
		return (String)tmpServiceVars.get(varName);
	}

	public String getSessionVariable(String varName){
		Hashtable tmpSessionVars = client.getSessionVariables();
		return (String)tmpSessionVars.get(varName);
	}

	public Hashtable getServerVariables(){
		return server.getServerVariables();
	}
	
	public long getTotalBytesSent(){
		return client.getTotalBytesSent();
	}

	public long getTotalBytesRecieved(){
		return client.getTotalBytesRecieved();
	}
	
	public Hashtable getApplicationVariables(){
		return service.getApplicationVariables();
	}

	public Hashtable getSessionVariables(){
		return client.getSessionVariables();
	}
	
	public ServiceList getServiceList()	{
		return server.getServiceList();
	}

}