package com.xfactorstudio.ahawasi;

import java.util.Vector;
import java.util.Hashtable;
/*
 * ServiceConfiguration.java
 *
 *
 * Copyright (C) 2001 Neeld Tanksley
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 

/**
 *
 *
 * The configuration object for a service
 *
 * @author Neeld Tanksley
 * @version .91
 */

 public class ServiceConfiguration {
  /**
  * Maximum number of clients to allow in a room at any time
  * 0 = unlimited
  * Default = 0
  */
  private int maxUsers = 0;
  
  /**
  * Maximum number of bytes to accept per message before booting client
  * Default is 10kb
  */
  private int maxMessageSize = 10000;
  
  /**
  * The port for this service to listen on
  */
  private int port = 0;
  
  /**
  * The name of this service
  */
  private String serviceName = "";
  
  /**
  *	The class name of this services MessageHandler
  */
  private String messageHandler = "";
  
  /**
  * The XMLElement containing the configuration for this service
  */
  private XMLElement serviceConfig;
  
  /**
  *	The Hashtable containing the application variables for this service
  */
  private Hashtable applicationVariables = new Hashtable();

  public ServiceConfiguration(XMLElement config) {
	this.serviceConfig = config;

	//First check for any attributes
	if(serviceConfig.getProperty("service_name") != null){
		setServiceName(serviceConfig.getProperty("service_name"));
	}
	if(serviceConfig.getProperty("port") != null){
		setPort(Integer.parseInt(serviceConfig.getProperty("port")));
	}
	if(serviceConfig.getProperty("max_users") != null){
		setMaxUsers(Integer.parseInt(serviceConfig.getProperty("max_users")));
	}
	if(serviceConfig.getProperty("max_messagesize") != null){
		setMaxMessageSize(Integer.parseInt(serviceConfig.getProperty("max_users")));
	}
	if(serviceConfig.getProperty("message_handler") != null){
		setMessageHandler(serviceConfig.getProperty("message_handler"));
	}

	//Now check for any children
	Vector childNodes = config.getChildren();

	for(int i = 0;i<childNodes.size();i++){
		XMLElement fooNode = (XMLElement)childNodes.get(i);
		String fooNodeName = fooNode.getTagName();
		String fooNodeValue = fooNode.getContents();

		if(fooNodeName.equalsIgnoreCase("service_name")){
			setServiceName(fooNodeValue);
		}
		if(fooNodeName.equalsIgnoreCase("port")){
			setPort(Integer.parseInt(fooNodeValue));
		}
		if(fooNodeName.equalsIgnoreCase("max_users")){
			setMaxUsers(Integer.parseInt(fooNodeValue));
		}
		if(fooNodeName.equalsIgnoreCase("max_messagesize")){
			setMaxMessageSize(Integer.parseInt(fooNodeValue));
		}
		if(fooNodeName.equalsIgnoreCase("message_handler")){
			setMessageHandler(fooNodeValue);
		}
		if(fooNodeName.equalsIgnoreCase("application_variable")){
			String appVarName = fooNode.getProperty("name");
			setApplicationVariable(appVarName,fooNodeValue);
		}
	}
  }
  public void setApplicationVariable(String name,String value){
    applicationVariables.put(name,value);
  }
  public Hashtable getApplicationVariables(){
   return this.applicationVariables;
  }
  public void setMessageHandler(String messageHandler){
    this.messageHandler = messageHandler;
  }
  public String getMessageHandler(){
    return this.messageHandler;
  }
  public void setMaxUsers(int maxUsers){
    this.maxUsers = maxUsers;
  }
  public void setMaxMessageSize(int maxSize) {
  	this.maxMessageSize = maxSize;
  }
  public int getMaxMessageSize() {
  	return this.maxMessageSize;
  }
  public int getMaxUsers(){
    return this.maxUsers;
  }
  public void setPort(int port){
    this.port = port;
  }
  public void setServiceName(String serviceName){
    this.serviceName = serviceName;
  }
  public int getPort(){
    return this.port;
  }
  public String getServiceName(){
    return this.serviceName;
  }
}