package com.xfactorstudio.ahawasi;
 
import java.util.*;
import java.io.*;
import java.net.Socket;
import java.net.*;

/*
 * SocketService.java
 *
 *
 * Copyright (C) 2001 Neeld Tanksley
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
 

/**
 *
 *
 * The Socket Service
 *
 * @author Neeld Tanksley
 * @version .91
 */

public class SocketService extends Thread {
	private ServerSocket serverSocket;
	private Thread tempThread;
	private ClientThread tempClient;
	private ClientList clientList= new ClientList();
	private boolean listening = true;
	private ServiceConfiguration config;
	private Ahawasi server;
	private int maxUsers;
	private int maxMessageSize;
	private int port;
	private String serviceName;
	private String messageHandler;
	private Hashtable applicationVariables;
	private XMLElement ConfXML;

  public SocketService(){

  }
  
	public  SocketService(XMLElement ConfXML,Ahawasi server){
		this.ConfXML = ConfXML;
		config = new ServiceConfiguration(ConfXML);
		this.server = server;
		this.maxUsers = config.getMaxUsers();
		this.port = config.getPort();
		this.maxMessageSize = config.getMaxMessageSize();
		this.serviceName = config.getServiceName();
		this.messageHandler = config.getMessageHandler();
		this.applicationVariables = config.getApplicationVariables();
	}

	public String getServiceName(){
		return config.getServiceName();
	}

	public void close(){
		this.removeAllClients();
		try{
			listening = false;
			serverSocket.close();
		}catch(SocketException se){
			se.printStackTrace(new PrintStream(server.err));
		}catch(IOException ioe){
			ioe.printStackTrace(new PrintStream(server.err));
		}catch(Exception e){
			e.printStackTrace(new PrintStream(server.err));
		}
	}

public synchronized void run(){
  server.serviceList.add(this);

  try {
    serverSocket = new ServerSocket(config.getPort());
  }catch(IOException e){
  	e.printStackTrace(new PrintStream(server.err));
	//server.err.println("why not");
    System.exit(-1);
  }
  server.out.println("Opening Room: " + serviceName + ", listening on port " + port);

	while (listening){
		try	{
		 Socket socket = serverSocket.accept();
	 
		 if((clientList.size() < maxUsers) || maxUsers == 0){
			tempClient = new ClientThread(socket,
			                              clientList,
			                              messageHandler,
			                              this,
			                              server);
									  
			addClient(tempClient);
			tempClient.start();
		 }else {
	        PrintWriter tmpout = new PrintWriter(socket.getOutputStream(), true);
	        tmpout.println("<control type=\"status\" code=\"500\" from=\"Ahawasi\" messvalue=\"user limit reached\" />");
	        tmpout.println('\u0000');
			tmpout.close();
			server.log.println("sent room full -- " + socket.getInetAddress());
			socket.close();
			
		 }
		 //end ss.accept() try
		}catch(SocketException se){
			se.printStackTrace(new PrintStream(server.err));
		}catch(IOException ioe){
			ioe.printStackTrace(new PrintStream(server.err));
		}catch(Exception e){
			e.printStackTrace(new PrintStream(server.err));
		}
	
	}

    try{
    	serverSocket.close();
    }catch(IOException e){
		e.printStackTrace(new PrintStream(server.err));
    }finally{
    server.setClosedServiceConfigs(this.ConfXML);
    server.serviceList.remove(this);
    serverSocket = null;
    tempThread = null;
    tempClient = null;
    clientList = null;
  }
}
	/**
	* return the current port
	*/
	public int getPort(){
		return this.port;
	}

	/**
	* return the number of users allowed to access this service
	*/
	public int getMaxUsers(){
		return this.maxUsers;
	}

	/**
	* return the maximum message size accepted
	* This helps prevent attempts to crash the server.
	* The client will be disconnected beyond this value
	*/
	public int getMaxMessageSize(){
		return this.maxMessageSize;
	}
	/**
	* return the current clientList
	*/
	public ClientList getClientList(){
		return this.clientList;
	}

	/**
	* This method loops through the client list for
	* the specified room and send everyone the current
	* message
	*/
	public void sendToAll(String message){
		int i;
		ClientThread foo;
		ClientThread tempClient;
		for (i=0;i<clientList.size();i++){
			foo = (ClientThread)clientList.get(i);
			if(foo != null){
				foo.send(message);
			}
		}
		message = null;
	}
	
	/**
	* This method adds a new client Thread to the client list
	*
	*/
	public void addClient(ClientThread tempClient){
		this.clientList.add(tempClient);
	}
 
 	/**
	* This method removes all clients
	*/
	public void removeAllClients(){
		for (int j=0;j<clientList.size();j++){
			ClientThread foo = (ClientThread)clientList.get(j);
			removeClient(foo);
			foo.kill();
		}
	}
 
	/**
	* This method removes a dead connection from the client list
	*/
	public void removeClient(ClientThread deadClient){
		int num;
		server.log.println("Disconnect -- "+deadClient.getClientIP());
		num=clientList.indexOf(deadClient);
		try{
			clientList.remove(num);
		}catch(Exception e){}
		sendToAll(clientList.toXML());
		deadClient.kill();
		deadClient = null;
	}
	
	/**
	* This method removes a dead connection from the client list
	*/
	public void removeClient(long userId){
		for (int j=0;j<clientList.size();j++){
			ClientThread foo = (ClientThread)clientList.get(j);
			if(foo.getUserId() == userId){
				removeClient(foo);
				foo.kill();
			}
		}
	}

	/**
	* This method sends the specified user the
	* private message
	*/
	public void sendTo(String message,long to){
		int i;
		ClientThread foo;
		ClientThread tempClient;
		for (i=0;i<clientList.size();i++){
			foo = (ClientThread)clientList.get(i);
			if (foo.getUserId() == to){
				foo.send(message);
			}
		}
		message = null;
	}
	/**
	* This method sends the specified user the
	* private message
	*/
	public void sendTo(String message,String userName){
		int i;
		ClientThread foo;
		ClientThread tempClient;
		for (i=0;i<clientList.size();i++){
			foo = (ClientThread)clientList.get(i);
			if (foo.getUserName().equals(userName)){
				foo.send(message);
			}
		}
		message = null;
	}
	
	/**
	* return the application variables Hashtable
	*/
	public Hashtable getApplicationVariables(){
		return this.applicationVariables;
	}
	
	/**
	* return the application variable specified by the key
	*/
	public String getApplicationVariable(String varName){
		return (String)this.applicationVariables.get(varName);
	}
	
	public ClientThread getClient(long clientId){
		ClientThread foo;
		ClientThread tempClient;
		for (int i=0;i<clientList.size();i++){
			foo = (ClientThread)clientList.get(i);
			if (foo.getUserId() == clientId){
				return foo;
			}
		}
		return null;
	}
	
	
	public String toXML(){
		String tmpXML = "";
		if (this.getApplicationVariable("Ahawasi:isVisible") == null || this.getApplicationVariable("Ahawasi:isVisible").equalsIgnoreCase("false")){
			tmpXML += "<service name=\"" + this.getServiceName() + "\" port=\"" + this.getPort() + "\" clients=\"" + this.getClientList().size() + "\"/>";
		}
		return tmpXML;
	}
	
	public String toXMLDetailed(){
		String tmpXML = "";
		if (this.getApplicationVariable("Ahawasi:isVisible") == null || this.getApplicationVariable("Ahawasi:isVisible").equalsIgnoreCase("false")){
			tmpXML += "<service name=\"" + this.getServiceName() + "\" port=\"" + this.getPort() + "\" clients=\"" + this.getClientList().size() + "\">";
			ClientList clientList = (ClientList)this.getClientList();
			for(int j=0;j<clientList.size();j++){
				ClientThread client = (ClientThread)clientList.get(j);
				tmpXML += "<client name=\"" + client.getUserName() + "\" id=\"" + client.getUserId() + "\" />";
			}
			tmpXML += "</service>";
		}
		return tmpXML;
	}
	
}